import * as _ from 'lodash';

const REPAIR_ENERGY_LIMIT = 500;

export function runTower(tower: StructureTower) {
  const room = tower.room;
  const hostiles = room.find(FIND_HOSTILE_CREEPS);
  if (hostiles.length > 0) {
    const hostile = _.minBy(hostiles, (h) => h.hits);
    if (hostile !== undefined) {
      const res = tower.attack(hostile);
      console.log('Attack', res);
      return;
    }
  }

  const creeps = room.find(FIND_MY_CREEPS, {
    filter: (c) => c.hits < c.hitsMax
  });
  if (creeps.length > 0) {
    const creep = _.minBy(creeps, (c) => c.hits);
    if (creep !== undefined) {
      const res = tower.heal(creep);
      console.log('Heal', res);
      return;
    }
  }

  // Energy only for attack and heal
  if (tower.store[RESOURCE_ENERGY] < REPAIR_ENERGY_LIMIT) {
    return;
  }

  const roads = room.find(FIND_STRUCTURES, {
    filter: (s) => s.structureType === STRUCTURE_ROAD && s.hits < s.hitsMax
  });
  if (roads.length > 0) {
    const road = _.minBy(roads, (r) => r.hits);
    if (road !== undefined) {
      const res = tower.repair(road);
      console.log('Repair', res);
      return;
    }
  }

  const structures = room.find(FIND_STRUCTURES, {
    filter: (s) => s.hits < s.hitsMax
  });
  if (structures.length > 0) {
    const structure = _.minBy(structures, (s) => s.hits);
    if (structure !== undefined) {
      const res = tower.repair(structure);
      console.log('Repair', res);
      return;
    }
  }
}
